//----------------------------------------------------------------------
// Titel     : DSP Heimradio mit ATTINY13A/85 fuer SI4735 Radio
//----------------------------------------------------------------------
// Funktion  : Misst Spannung via Poti und ermittelt Stufen zur
//				 fixen Frequenzstufenwahl
// Schaltung : siehe B. Kainka
// Dateiname : DSP_heimradio1.c
// Includes  : si4735.h (i2cmaster.h innerhalb si4735.h)
// Hinweis   : i2cmaster.S (Assem. Source) den source Files hinzuf.
//----------------------------------------------------------------------
// Prozessor : Attiny13A / Attiny85
// Takt       : 1.00 MHz (RC Oszillator 8MHz, CLKDIV8 aktiviert)
// Sprache   : C
// Datum     : 30.04.2014 22:28:33
// Version   : 1.14
// Autor     : Andreas Nestler
// Changes   : 
//		v1   : initial version; Si4735 is not commandable
//		v1.01: i2_start_wait statt i2c_start fr si4735init,
//             da das IC mehr Zeit braucht, bevor die folgenden Daten
//             geschrieben werden knnen. Die folgenden Zugriffe sind
//             nicht betroffen	
//      v1.02: MONO operation mode forced within si4735init routine
//      v1.03: MONO opMode removed,
//             calculation for channel selection changed N++ instead N--
//      v1.04: Property check with Volume control if button is toggled 
//             in OpMode
//      v1.05: FM_RSQ_STATUS added
//      v1.06: Some Properties added
//      v1.07: si4735.c and.h includes
//		v1.08: header and c files: includes updated
//		v1.09: Channel selection issue solved -> pointer to word issue
//      v1.10: OLED I2C Display support
//      v1.11: code optimization for less space, avoid global vars
//      v1.12: compiler switch optimize, logo disabled, SNR data output
//      v1.13: Display update only each 255 cycle, 
//             own int to ascii routine to speed up and
//             to avoid old data fragments
//      v1.14: key functions used from PeDa, code clean-up header files
//             Display data now for PRog and OP mode, added displ funcs 
//----------------------------------------------------------------------
// Pinconfig:
// =====================
// PB0 -> SDA	 (O)	-> TWI Daten
// PB1 -> Taste  (I) PU	-> Taste an GND
// PB2 -> SCL	 (O)	-> TWI Takt  
// PB3 -> unused (I) 	-> auf GND legen,
// PB4 -> ADC2   (AI) 	-> Potischleifer
// PB5 -> unused (I) PU -> reserviert als RESET-Pin
//
// Legende: I = Input , O = Output, AI = Analog Input, PU = Pull-up
//
//----------------------------------------------------------------------

/* ! Debug Control in globals.h ! */
#include "inc/globals.h"   // before delay.h, da F_CPU benoetigt wird
#include <util/delay.h>    // for _delay functions, requires F_CPU definition
#include "inc/i2cmaster.h" // for i2c functions
#include "inc/si4735.h"	   // si4735 header with function protos
#include "inc/display.h"   // OLED display header with function protos
#include "inc/keys.h"      // keys header with function protos
#include "inc/numtostr.h"  // Zahl zu String Konvertierung

#include <inttypes.h>      // Allgemeine Bibliotheken
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
// #include <stdlib.h> // fuer conversion functions utoa, itoa etc

/*
** constant definitions
*/
#define F1 8890			// Freq.: N-Joy
#define F2 9350			// Freq.: NDR2
#define F3 9730			// Freq.: Antenne-MV
#define F4 10480		// Freq.: Ostseewelle
#define F5 10650		// Freq.: DLF

/*
** Default Frequenzen inkl. Endemarke in EEPROM
** muss global definiert werden!
*/
int EEMEM Freq_list[6]={F1, F2, F3, F4, F5, 0xFFFF};

/*
** globale Variablen
*/
#ifdef DISPLAY_SUPPORT
static char str1[4];  // char array for 3 digits + string terminator
                      // used to display RSQ data on OLED
// global definition for strings means here less memory
static char oledString1[]="RSSI:    ";
static char oledString2[]="SNR:     ";
static char oledString3[]="STBLEND: ";
// static char oledString4[]="FREQ:    ";
// static char oledString5[]="STORE:   ";

#endif					  

/*
** function prototypes
*/ 
void ADC_Init(void); // ADC initialization 
unsigned int ADC_Read(unsigned char channel); // ADC read function
void Display_Text_OPmode(void);
void Display_Text_PROGmode(void);
void Display_RSQ(tRSQ _rsq, unsigned char cNo_elems); // display RSQ data in display
// void Display_PRGData(unsigned int _iFreq, unsigned char _store);
void Display_FreqCh_Data(unsigned int _iFreq, unsigned char _store);



int main(void)
{
    unsigned char cN, cNmax, cCh=0, cCh_old;
	unsigned char cButPush_Flag=0, cButPush_long_Flag=0;
	unsigned int iFreq, iPoti, iDiv, * pUint_EEaddr;
	#ifdef _DEBUG_RSQ_RSSI_GPO
	   unsigned char cT; // cT only in _DEBUG_RSQ_RSSI_GPO mode
	#endif
	#ifdef DISPLAY_SUPPORT
	  unsigned int iI=DISPLAY_REFRESH_CYC; // Display refresh after N cycles
	  unsigned char cNelems;  // no of array elements -> to display data on OLED
	#endif
	#ifdef READ_RSQ_DATA
	  unsigned char cRSQ_read_Flag=0; // RSQ data read flag only valid if RSQ data read
	  tRSQ rsq;
	#endif
	
		
    /* Portpin configuration */    
	DDRB = (1<<DDB0) | (1<<DDB2);	// Portpins B.0 und B.2 auf Ausgang,
    PORTB = (1<<PB0) | (1<<PB2);	// und diese Bits auf 1
	PORTB |= (1<<PB1) | (1<<PB5);	// noch Pull-up f. Inputs PB1 u. PB5 ein
	DIDR0 = (1<<DDB4);				// Disable dig. input buffer for PB4->ADC2
									// this saves current consumption
//	KEY_PORT |= 1<<KEY0;                                  // Pullup on								

    /* Timer config for keys functions */
    TCCR0A = 1<<WGM01;              // CTC mode
	TCCR0B = 1<<CS02^1<<CS00;		// divide by 1024
	OCR0A = F_CPU / 1024.0 * 10e-3 - 0.5;   // 10ms interrupt
//	TIMSK = 1<<TOIE0;				// enable timer interrupt	
    #if (__AVR_ATtiny85__ == 1) || (__AVR_ATtiny45__ == 1) || (__AVR_ATtiny25__ == 1)
      TIMSK = 1<<OCIE0A;			//  TIMSK for ATTINY 25/45/85							
	#elif (__AVR_ATtiny13A__ == 1)
	  TIMSK0 = 1<<OCIE0A;           // TIMSK0 for ATTINY13,
	#endif 
	sei();                          // enable interrupts
									
									
	/* call init functions */
	ADC_Init();
	i2c_init();
	_delay_ms(500);
	#ifdef DISPLAY_SUPPORT
	  setup_OLED();
	  cNelems = sizeof(str1)/sizeof(str1[0]); // Anzahl der Arrayelemente
	#endif // #if DISPLAY_SUPPORT	  
	
	si4735init();
	fm_deemphasis();				// set correct european FM deemphasis of 50us
	
	
	/* programming or OP mode */
    if( get_key_press( 1<<KEY0 )) { // Programmier-Modus wenn Taste
									// beim Einschalten gedrckt
	   do 
	   {
	   } while (!( get_key_release( 1<<KEY0 ))); // warten bis Taste losgelassen
      
	                                // beim Einschalten gedrckt
	  #ifdef DISPLAY_SUPPORT
	    Display_Text_PROGmode();								
	  #endif
	  								
 	  iFreq=8700;                   // Frequenz auf FM-Bandanfang
	  // fm_seek_tune_snr_threshold(); // SNR threshold for seek 6dB (3dB def)
	  // fm_seek_tune_rssi_threshold(); // 0x14 -> default
	  // fm_seek_freq_spacing();		// frequency spacing for seek 50kHz (100kHz def)
	  si4735freq(iFreq);			// Frequenz auf UKW-Bandanfang einstellen
	  _delay_ms(200);
	  pUint_EEaddr=0;				// setze Pointer auf 0 -> EEAddr 0
	//  SETBIT(cButPush_Flag,0);      // Flag erstmalig setzen, 
	                                // damit gleich im Prog Mode der Suchlauf startet
	  Scan();						// Neuen kanal suchen - erstmalig
	  cN=1;							// cN Speicherkanal fuer gefundenen Sender -> zuerst auf 1 setzen
	  for (;;) { 					// program loop forever
		if( get_key_short( 1<<KEY0 )) { // Neuen kanal suchen, immer wenn Taste kurz gedrckt wird
			Scan();  // Neuen kanal suchen
		}
			
	    if( get_key_long( 1<<KEY0 )) {              // langer Tastendruck -> Speichern
		  iFreq = Fm_tune_status();
          eeprom_update_word(pUint_EEaddr,iFreq);
		  #ifdef DISPLAY_SUPPORT
		    // Display_PRGData(iFreq,cN++);
			Display_FreqCh_Data(iFreq,cN++); // Freq and channel output
		  #endif
          pUint_EEaddr ++;		     				// Pointer-Inkrement
          iFreq = 0xFFFF;                           // Endemarke
          eeprom_update_word(pUint_EEaddr,iFreq);
		}; // end if
      } // end of program loop forever
	}; // end if BUTTON_PUSHED; programming mode
	
  // Start Opmode
  // Anzahl gespeicherter Kanle auslesen -> Nmax
  // cN = 0; // was in original BASCOM file, obsolete
  #ifdef DISPLAY_SUPPORT
    Display_Text_OPmode();
  #endif
  cNmax = 0;
  pUint_EEaddr = 0;
  do {
    iFreq=eeprom_read_word(pUint_EEaddr);
	if (iFreq < 10800) cNmax++;
	pUint_EEaddr++;
  } while(iFreq != 0xFFFF);

//  if (cNmax == 0) Scan();  // falls keine Sender gespeichert, einfach scannen
  cN = cNmax * 2;
  cN--;
  iDiv = 1024 / cN;
 // cCh = 0;
  cCh_old = 255;
  for (;;) {						// loop forever
    CLEARBIT(cButPush_long_Flag,0); // reset flag
	/* Im Opmode hat die Taste folgende Funktionen:
	-----------------------------------------------
	  Taste kurz gedrckt:
	   -> Mono Control,
	   -> cButPush_Flag toggeln
	  Taste >1s gedrckt 
	   -> RSQ Daten lesen,
	   -> als Pulsbreite value x 1ms auf GPO1,
	   -> Daten in EEPROM ab Addr. 100 wegschreiben
	   -> cButPush_long_Flag setzen/lschen
	-----------------------------------------------   */
  /*  if BUTTON_PUSHED {				// Tastenabfrage im OpMode
	  do {							// Test auf Taste gedrueckt
	    _delay_ms(10);				// Tasten-Entprellung
	  } while (BUTTON_PUSHED); */
   if( get_key_short( 1<<KEY0 )) {
      TOGGLEBIT(cButPush_Flag,0);
   }; //end if
	 if( get_key_long( 1<<KEY0 )) {
	    SETBIT(cButPush_long_Flag,0);               // Flag setzen, wenn Tastendruck > 1s
	 }
	    else CLEARBIT(cButPush_long_Flag,0);          // andernfalls Flag zurcksetzen */

	_delay_ms(1);
	Mono_Ctrl(cButPush_Flag);		// call MonoCtrl function
    iPoti = ADC_Read(2);			// ADC single conversion for ch ADC2 
    iPoti = iPoti / iDiv;
    if (!(CHECKBIT(iPoti,0))) {     // Check if Bit 0 of iPoti is cleared 
      cCh = iPoti / 2;			    // -> Hyterese
    }; // end if
    if (cCh != cCh_old) {  // Neue Frequenz einstellen, wenn Kanal gendert wurde
      pUint_EEaddr = PUINTCAST(2*cCh);// cCh*2 due to byte - word access issue
      iFreq=eeprom_read_word(pUint_EEaddr);
      si4735freq(iFreq);
      cCh_old = cCh;
    }; // end if
	if (cButPush_long_Flag) { /* RSQ Auswertung nur bei langem Tastendruck */
	  #ifdef READ_RSQ_DATA
	    rsq = Fm_rsq_status();
	    cRSQ_read_Flag = 1;  // set flag if rsq is read at least once
	  #endif
	  
	  #ifdef _DEBUG_RSQ_RSSI_GPO
	    GPO_set(0x02); // Output High
	    for (cT=0;cT<=rsq.rssi;cT++)
	    {
		    _delay_ms(1);
	    }
	    GPO_set(0x00); //Output Low
	  #endif // _DEBUG_RSQ_RSSI_GPO
	  
	  #if _DEBUG >= 1  // Debug level 1
	    // Schreibe RSQ Daten ins EEPROM ab Addresse 100
	    pUint_EEaddr=PUINTCAST(100);
	    eeprom_update_word(pUint_EEaddr,rsq.rssi);
	    pUint_EEaddr++;
	    eeprom_update_word(pUint_EEaddr,rsq.snr);
	    pUint_EEaddr++;
	    eeprom_update_word(pUint_EEaddr,rsq.stblend);
	    pUint_EEaddr++;
	  #endif // #if _DEBUG >= 1
	  
	  #if _DEBUG >= 2 // Debug level 2, more data in EEPROM
	    eeprom_update_word(pUint_EEaddr,0xFFFF);
	    pUint_EEaddr++;
	    eeprom_update_word(pUint_EEaddr,cNmax);
	    pUint_EEaddr++;
	    eeprom_update_word(pUint_EEaddr,cCh);
	    pUint_EEaddr++;
	    eeprom_update_word(pUint_EEaddr,ADC_Read(2));
	    pUint_EEaddr++;
	    eeprom_update_word(pUint_EEaddr,0xFFFF);
	    pUint_EEaddr++;
	    eeprom_update_word(pUint_EEaddr,(unsigned int) PUINTCAST(cCh));
	    pUint_EEaddr++;
	    eeprom_update_word(pUint_EEaddr,iFreq);
	  #endif // #if _DEBUG >= 2
	  	
	}; // if (cButPush_long_Flag)
	#ifdef DISPLAY_SUPPORT
	  if (cRSQ_read_Flag  && iI==DISPLAY_REFRESH_CYC) {  // nur Ausgabe, wenn auch die RSQ Daten einmal gelesen wurden 
		Display_FreqCh_Data(iFreq,cCh+1); // Freq and channel output, +1 in order to start with 1
        Display_RSQ(rsq,cNelems);    // und iI auf counter Resetwert ist
	   // iI--;
	    if (!(--iI)) iI=DISPLAY_REFRESH_CYC;  // Counter dekrementieren und reset if zero
	  }
	  else if (cRSQ_read_Flag) {          // Counter dekrementieren wenn cRSQ_read_Flag gesetzt
	   // iI--;
	    if (!(--iI)) iI=DISPLAY_REFRESH_CYC;  // Counter dekrementieren und reset if zero
	  } // if (cRSQ_read_Flag)
	#elif _DEBUG >= 1 || defined _DEBUG_RSQ_RSSI_GPO // no DISPLAY_SUPPORT AND _DEBUG >= 1
	  UNUSED(cRSQ_read_Flag); // only set but not read if no display support
	#endif // #if DISPLAY_SUPPORT

  } // end of loop forever
  return 0; // main return value
} // end of main


/* Initialize the ADC */
void ADC_Init(void)
{

  // die Versorgungsspannung AVcc als Referenz whlen - default:
  // REFS0 = 0; REFS1 = 0

  // Bit ADFR ("free running") in ADCSRA steht beim Einschalten
  // schon auf 0, also single conversion

  ADCSRA = (1<<ADPS2) ;     // Frequenzvorteiler 1:16 fuer 50kHz < fadc < 200kHz
  ADCSRA |= (1<<ADEN);      // ADC aktivieren
 
  /* nach Aktivieren des ADC wird ein "Dummy-Readout" empfohlen, man liest
     also einen Wert und verwirft diesen, um den ADC "warmlaufen zu lassen" */
 
  ADCSRA |= (1<<ADSC);                  // eine ADC-Wandlung 
  while (ADCSRA & (1<<ADSC) ) {         // auf Abschluss der Konvertierung warten
  }
  /* ADCW muss einmal gelesen werden, sonst wird Ergebnis der nchsten
     Wandlung nicht bernommen. */
  (void) ADCW;
	
}


/* ADC Einzelmessung from channel */
unsigned int ADC_Read( unsigned char channel )
{
  // Kanal waehlen, ohne andere Bits zu beeinflussen
  ADMUX = (ADMUX & ~(0x03)) | (channel & 0x03);
  ADCSRA |= (1<<ADSC);            // eine Wandlung "single conversion"
  while (ADCSRA & (1<<ADSC) );    // auf Abschluss der Konvertierung warten
  return ADCW;                    // ADC auslesen und zurckgeben
}

/* display RSQ data in display */
#ifdef DISPLAY_SUPPORT

void Display_Text_OPmode(void)
{
	sendStrXY(0,0,"DSP-Radio - RUN ");
	sendStrXY(1,0,"****************");
	sendStrXY(3,0,"Ch   F    .  MHz"); // Freq
	sendStrXY(4,0,oledString1); // RSSI
	sendStrXY(5,0,oledString2); // SNR
	sendStrXY(6,0,oledString3); // STBLEND

	ssd1306_command(SSD1306_DISPLAYON);			 // set display on
}

void Display_Text_PROGmode(void)
{
	sendStrXY(0,0,"DSP-Radio - PRG ");
	sendStrXY(1,0,"****************");
	sendStrXY(3,0,"Ch   F    .  MHz"); // Freq
	// sendStrXY(3,0,oledString4);
	// sendStrXY(4,0,oledString5);
	// sendStrXY(6,1,oledString6);
	
	ssd1306_command(SSD1306_DISPLAYON);			 // set display on
}

void Display_RSQ(tRSQ _rsq, unsigned char cNo_elems)  
{
	_my_uchar2bcd(_rsq.rssi,str1,cNo_elems,1);
	sendStrXY(4,9,str1);
	_my_uchar2bcd(_rsq.snr,str1,cNo_elems,1);
	sendStrXY(5,9,str1);
	_my_uchar2bcd((_rsq.stblend & 0x7F),str1,cNo_elems,1);
	sendStrXY(6,9,str1);
}

/* void Display_PRGData(unsigned int _iFreq, unsigned char _store)
{
	char _str1[5];
	
	utoa(_iFreq,_str1,10);
	sendStrXY(3,10,_str1);
	_my_uchar2bcd(_store,_str1,5);
	sendStrXY(4,10,_str1);
} */


/* die folgende Funktion funktioniert und ist allgemeingueltig */
/* Der Umgang mit float bedeutet aber zustzlich >1000 Byte Code */
/* void Display_FreqCh_Data(unsigned int _iFreq, unsigned char _store)
{
	char _str1[4], _str2[7];
	float fm_freq;

    fm_freq=_iFreq;
	fm_freq /= 100;
	// dtostrf(fm_freq,6,2,_str2);
	// ftoa(_str2,fm_freq,2,3,1);
	ftoa(_str2,fm_freq,3);
	sendStrXY(3,7,_str2);
	_my_uchar2bcd(_store,_str1,2);
	sendStrXY(3,3,_str1);
} */

void Display_FreqCh_Data(unsigned int _iFreq, unsigned char _store)
{
	char _str1[4];
	unsigned char _fm_freq;

    /* Um unntigen Code zu vermeiden, werden einfach die Vorkomma- und Nachkommastellen */
	/* berechnet und dann an bestimten Positionen ausgegeben */
	_fm_freq=_iFreq / 100; // Vorkommastellen berechnen
	_my_uchar2bcd(_fm_freq,_str1,4,1); // in BCD zur Darstellung wandeln
	                                   // 3 Stellen + Null Terminator
	sendStrXY(3,7,_str1); // Ausgabe der Vorkommastellen
	_fm_freq=_iFreq % 100; // Nachkommastellen berechnen
	 // utoa(_fm_freq,_str1,10); // utoa statt eigener Funktion, da ich die fhrende Null bentige
	_my_uchar2bcd(_fm_freq,_str1,3,0);  // in BCD zur Darstellung wandeln,
	                                    // 2 Stellen + Null Terminator, fhrende Nullen
	sendStrXY(3,11,_str1); // Ausgabe der Vorkommastellen
	_my_uchar2bcd(_store,_str1,2,1);  // Kanal in BCD zur Darstellung wandeln
	                                  // 1 Stelle + Null Terminator
	sendStrXY(3,3,_str1); // Ausgabe des Kanals
}


#endif // DISPLAY_SUPPORT

