//----------------------------------------------------------------------
// Titel     : DSP Heimradio mit ATTINY13A/85 fuer SI4735 Radio
//----------------------------------------------------------------------
// Funktion  : Misst Spannung via Poti und ermittelt Stufen zur
//				 fixen Frequenzstufenwahl
// Schaltung : siehe B. Kainka
// Dateiname : DSP_heimradio1.c
// Includes  : si4735.h (i2cmaster.h innerhalb si4735.h)
// Hinweis   : i2cmaster.S (Assem. Source) den source Files hinzuf.
//----------------------------------------------------------------------
// Prozessor : Attiny13A / Attiny85
// Takt       : 1.00 MHz (RC Oszillator 8MHz, CLKDIV8 aktiviert)
// Sprache   : C
// Datum     : 24.04.2014 22:18:33
// Version   : 1.13
// Autor     : Andreas Nestler
// Changes   : 
//		v1   : initial version; Si4735 is not commandable
//		v1.01: i2_start_wait statt i2c_start fr si4735init,
//             da das IC mehr Zeit braucht, bevor die folgenden Daten
//             geschrieben werden knnen. Die folgenden Zugriffe sind
//             nicht betroffen	
//      v1.02: MONO operation mode forced within si4735init routine
//      v1.03: MONO opMode removed,
//             calculation for channel selection changed N++ instead N--
//      v1.04: Property check with Volume control if button is toggled 
//             in OpMode
//      v1.05: FM_RSQ_STATUS added
//      v1.06: Some Properties added
//      v1.07: si4735.c and.h includes
//		v1.08: header and c files: includes updated
//		v1.09: Channel selection issue solved -> pointer to word issue
//      v1.10: OLED I2C Display support
//      v1.11: code optimization for less space, avoid global vars
//      v1.12: compiler switch optimize, logo disabled, SNR data output
//      v1.13: Display update only each 255 cycle, 
//             own int to ascii routine to speed up and
//             to avoid old data fragments
//----------------------------------------------------------------------
// Pinconfig:
// =====================
// PB0 -> SDA	 (O)	-> TWI Daten
// PB1 -> Taste  (I) PU	-> Taste an GND
// PB2 -> SCL	 (O)	-> TWI Takt  
// PB3 -> unused (I) 	-> auf GND legen,
// PB4 -> ADC2   (AI) 	-> Potischleifer
// PB5 -> unused (I) PU -> reserviert als RESET-Pin
//
// Legende: I = Input , O = Output, AI = Analog Input, PU = Pull-up
//
//----------------------------------------------------------------------

#include "inc/si4735.h"	   // add before delay.h, include F_CPU definition
#include "inc/display.h"

#include <inttypes.h>      // Allgemeine Bibliotheken
#include <avr/io.h>
#include <avr/interrupt.h>
// #include <util/delay.h> // for _delay_ms() function, already included in si4735.h
#include <avr/eeprom.h>
#include <stdlib.h>
#include "inc/xitoa.h"

/*
** constant definitions
*/
#define F1 8890			// Freq.: N-Joy
#define F2 9350			// Freq.: NDR2
#define F3 9730			// Freq.: Antenne-MV
#define F4 10480		// Freq.: Ostseewelle
#define F5 10650		// Freq.: DLF

/*
** Default Frequenzen inkl. Endemarke in EEPROM
** muss global definiert werden!
*/
int EEMEM Freq_list[6]={F1, F2, F3, F4, F5, 0xFFFF};

/*
** globale Variablen
*/
// tRSQ rsq;							/* deklariert Variable des Typs tRSQ */
// unsigned char ready=0;
/* avoid global variables as far as possible to avoid side effects and to save memory */

// char oledString1[]="RSSI:";
// char oledString2[]="STBLEND:";
static char str1[4];  // char array for 3 digits + string terminator

/*
** function prototypes
*/ 
void ADC_Init(void); // ADC initialization 
unsigned int ADC_Read(unsigned char channel); // ADC read function
void Display_RSQ(tRSQ _rsq, unsigned char cNo_elems); // display RSQ data in display



int main(void)
{
    unsigned char cT, cN, cNmax, cCh=0, cCh_old;
	unsigned char cButPush_Flag=0, cButPush_long_Flag=0, cRSQ_read_Flag=0, cNelems;
	unsigned int iFreq, iPoti, iDiv, * pUint_EEaddr;
	#ifdef DISPLAY_SUPPORT
	  unsigned int iI=DISPLAY_REFRESH_CYC;
	#endif
	
	#if defined (_DEBUG_RSQ_RSSI_GPO) || defined (DISPLAY_SUPPORT)
	  tRSQ rsq;
	#elif  _DEBUG >= 1
	  tRSQ rsq;
	#endif

/*	#ifdef DISPLAY_SUPPORT
	char str1[5];
	#endif */
		
    /* Portpin configuration */    
	DDRB = (1<<DDB0) | (1<<DDB2);	// Portpins B.0 und B.2 auf Ausgang,
    PORTB = (1<<PB0) | (1<<PB2);	// und diese Bits auf 1
	PORTB |= (1<<PB1) | (1<<PB5);	// noch Pull-up f. Inputs PB1 u. PB5 ein
	DIDR0 = (1<<DDB4);				// Disable dig. input buffer for PB4->ADC2
									// this saves current consumption
	/* call init functions */
	ADC_Init();
	i2c_init();
	_delay_ms(500);
	#ifdef DISPLAY_SUPPORT
	/*  init_OLED();
	  clear_display();
	  setXY(3,1);
	  sendStr("RSSI:"); */
	  setup_OLED();
	  cNelems = sizeof(str1)/sizeof(str1[0]); // Anzahl der Arrayelemente
	  #endif // #if DISPLAY_SUPPORT	  
	
	si4735init();
	fm_deemphasis();				// set correct FM deemphasis of 50us
	
	
	/* endless loop for programming mode */
    if BUTTON_PUSHED {				// Programmier-Modus wenn Taste 
	  do {							// beim Einschalten gedrckt
	    _delay_ms(10);				// Tasten-Entprellung
	  } while (BUTTON_PUSHED);
	  iFreq=8700;
	  fm_seek_tune_snr_threshold(); // SNR threshold for seek
	 // fm_seek_tune_rssi_threshold(); // 0x14 -> default
	  fm_seek_freq_spacing();		// frequency spacing for seek
	  si4735freq(iFreq);			// Frequenz auf UKW-Bandanfang einstellen
	  _delay_ms(200);
	  pUint_EEaddr=0;				// setze Pointer auf 0 -> EEAddr 0
	  for (;;) { 					// program loop forever
	    Scan();						// Neuen kanal suchen
		do {
	      _delay_ms(10);
		} while (!BUTTON_PUSHED);
		cT=0;						// cT initialisieren
		do {
		  _delay_ms(10);
		  cT++;   									// Lnge des Tastenducks messen -> 10ms inkrement
		  if (cT > 200) cT=200;						// Limitieren auf 2s -> verhindern eines overflow
		} while (BUTTON_PUSHED);
		if (cT > 100) {                             // lnger 1 s: Speichern
          iFreq = Fm_tune_status();
          eeprom_update_word(pUint_EEaddr,iFreq);
          // 'Print #1 , F , N */
          pUint_EEaddr ++;		     				// Pointer-Inkrement
          iFreq = 0xFFFF;                           // Endemarke
          eeprom_update_word(pUint_EEaddr,iFreq);
		}; // end if
      } // end of program loop forever
	}; // end if BUTTON_PUSHED; programming mode
	
  // Start Opmode
  // Anzahl gespeicherter Kanle auslesen -> Nmax
  // cN = 0; // was in original BASCOM file, obsolete
  cNmax = 0;
  pUint_EEaddr = 0;
  do {
    iFreq=eeprom_read_word(pUint_EEaddr);
	if (iFreq < 10800) cNmax++;
	pUint_EEaddr++;
  } while(iFreq != 0xFFFF);

//  if (cNmax == 0) Scan();  // falls keine Sender gespeichert, einfach scannen
  cN = cNmax * 2;
  cN--;
  iDiv = 1024 / cN;
 // cCh = 0;
  cCh_old = 255;
  for (;;) {						// loop forever
    CLEARBIT(cButPush_long_Flag,0); // reset flag
	/* Im Opmode hat die Taste folgende Funktionen:
	-----------------------------------------------
	  Taste kurz gedrckt:
	   -> Mono Control,
	   -> cButPush_Flag toggeln
	  Taste >1s gedrckt 
	   -> RSQ Daten lesen,
	   -> als Pulsbreite value x 1ms auf GPO1,
	   -> Daten in EEPROM ab Addr. 100 wegschreiben
	   -> cButPush_long_Flag setzen/lschen
	-----------------------------------------------   */
    if BUTTON_PUSHED {				// Tastenabfrage im OpMode
	  do {							// Test auf Taste gedrueckt
	    _delay_ms(10);				// Tasten-Entprellung
	  } while (BUTTON_PUSHED); 
      TOGGLEBIT(cButPush_Flag,0);
	  do {
		_delay_ms(10);
	  } while (!BUTTON_PUSHED);
	  cT=0;						    // cT initialisieren
	  do {
		_delay_ms(10);
		cT++;   									// Lnge des Tastenducks messen -> 10ms inkrement
		if (cT > 200) cT=200;						// Limitieren auf 2s -> verhindern eines overflow
	  } while (BUTTON_PUSHED);
	  if (cT > 100) {
	    SETBIT(cButPush_long_Flag,0);               // Flag setzen, wenn Tastendruck > 1s
	  }
	  else CLEARBIT(cButPush_long_Flag,0);          // andernfalls Flag zurcksetzen
	}; //end if 
	_delay_ms(1);
	Mono_Ctrl(cButPush_Flag);		// call MonoCtrl function
    iPoti = ADC_Read(2);			// ADC single conversion for ch ADC2 
    iPoti = iPoti / iDiv;
    if (!(CHECKBIT(iPoti,0))) {     // Check if Bit 0 of iPoti is cleared 
      cCh = iPoti / 2;			    // -> Hyterese
    }; // end if
    if (cCh != cCh_old) {  // Neue Frequenz einstellen, wenn Kanal gendert wurde
      pUint_EEaddr = PUINTCAST(2*cCh);// cCh*2 due to byte - word access issue
      // Readeeprom F , N
      iFreq=eeprom_read_word(pUint_EEaddr);
      si4735freq(iFreq);
      cCh_old = cCh;
    }; // end if
	if (cButPush_long_Flag) { /* RSQ Auswertung nur bei langem Tastendruck */
	  #if defined (_DEBUG_RSQ_RSSI_GPO) || defined (DISPLAY_SUPPORT)
	    rsq = Fm_rsq_status();
	    cRSQ_read_Flag = 1;  // set flag if rsq is read at least once
	  #elif  _DEBUG >= 1
	    rsq = Fm_rsq_status();
	    cRSQ_read_Flag = 1;  // set flag if rsq is read at least once
	  #endif
	  
	  #ifdef _DEBUG_RSQ_RSSI_GPO
	  GPO_set(0x02); // Output High
	  for (cT=0;cT<=rsq.rssi;cT++)
	  {
	  	_delay_ms(1);
	  }
	  GPO_set(0x00); //Output Low
	  #endif // _DEBUG_RSQ_RSSI_GPO
	  
	  #if _DEBUG >= 1  // Debug level 1
	  // Schreibe RSQ Daten ins EEPROM ab Addresse 100
	  pUint_EEaddr=PUINTCAST(100);
	  eeprom_update_word(pUint_EEaddr,rsq.rssi);
	  pUint_EEaddr++;
	  eeprom_update_word(pUint_EEaddr,rsq.snr);
	  pUint_EEaddr++;
	  eeprom_update_word(pUint_EEaddr,rsq.stblend);
	  pUint_EEaddr++;
	  #endif // #if _DEBUG >= 1
	  
	  #if _DEBUG >= 2 // Debug level 2, more data in EEPROM
	  eeprom_update_word(pUint_EEaddr,0xFFFF);
	  pUint_EEaddr++;
	  eeprom_update_word(pUint_EEaddr,cNmax);
	  pUint_EEaddr++;
	  eeprom_update_word(pUint_EEaddr,cCh);
	  pUint_EEaddr++;
	  eeprom_update_word(pUint_EEaddr,ADC_Read(2));
	  pUint_EEaddr++;
	  eeprom_update_word(pUint_EEaddr,0xFFFF);
	  pUint_EEaddr++;
	  eeprom_update_word(pUint_EEaddr,(unsigned int) PUINTCAST(cCh));
	  pUint_EEaddr++;
	  eeprom_update_word(pUint_EEaddr,iFreq);
	  #endif // #if _DEBUG >= 2
	  	
	}; // if (cButPush_long_Flag)
	#ifdef DISPLAY_SUPPORT
	if (cRSQ_read_Flag  && iI==DISPLAY_REFRESH_CYC) {  // nur Ausgabe, wenn auch die RSQ Daten einmal gelesen wurden 
      Display_RSQ(rsq,cNelems);    // und iI auf counter Resetwert ist
	 // iI--;
	  if (!(--iI)) iI=DISPLAY_REFRESH_CYC;  // Counter dekrementieren und reset if zero
	}
	else if (cRSQ_read_Flag) {          // Counter dekrementieren wenn cRSQ_read_Flag gesetzt
	 // iI--;
	  if (!(--iI)) iI=DISPLAY_REFRESH_CYC;  // Counter dekrementieren und reset if zero
	} // if (cRSQ_read_Flag)
	#endif // #if DISPLAY_SUPPORT

  } // end of loop forever
  return 0; // main return value
} // end of main


/* Initialize the ADC */
void ADC_Init(void)
{

  // die Versorgungsspannung AVcc als Referenz whlen - default:
  // REFS0 = 0; REFS1 = 0

  // Bit ADFR ("free running") in ADCSRA steht beim Einschalten
  // schon auf 0, also single conversion

  ADCSRA = (1<<ADPS2) ;     // Frequenzvorteiler 1:16 fuer 50kHz < fadc < 200kHz
  ADCSRA |= (1<<ADEN);      // ADC aktivieren
 
  /* nach Aktivieren des ADC wird ein "Dummy-Readout" empfohlen, man liest
     also einen Wert und verwirft diesen, um den ADC "warmlaufen zu lassen" */
 
  ADCSRA |= (1<<ADSC);                  // eine ADC-Wandlung 
  while (ADCSRA & (1<<ADSC) ) {         // auf Abschluss der Konvertierung warten
  }
  /* ADCW muss einmal gelesen werden, sonst wird Ergebnis der nchsten
     Wandlung nicht bernommen. */
  (void) ADCW;
	
}


/* ADC Einzelmessung from channel */
unsigned int ADC_Read( unsigned char channel )
{
  // Kanal waehlen, ohne andere Bits zu beeinflussen
  ADMUX = (ADMUX & ~(0x03)) | (channel & 0x03);
  ADCSRA |= (1<<ADSC);            // eine Wandlung "single conversion"
  while (ADCSRA & (1<<ADSC) );    // auf Abschluss der Konvertierung warten
  return ADCW;                    // ADC auslesen und zurckgeben
}

/* display RSQ data in display */
void Display_RSQ(tRSQ _rsq, unsigned char cNo_elems)  
{
	// char str1[4]; // char array for 3 digits + string terminator
	// unsigned char nelems;
	
	// nelems = sizeof(str1)/sizeof(str1[0]); // Anzahl der Arrayelemente
	
	// my_uitoa(_rsq.rssi,str1);
	_my_uchar2bcd(_rsq.rssi,str1,cNo_elems);
	//setXY(2,10);
	sendStrXY(2,10,str1);
	_my_uchar2bcd(_rsq.snr,str1,cNo_elems);
	//setXY(4,10);
	sendStrXY(4,10,str1);
	_my_uchar2bcd((_rsq.stblend & 0x7F),str1,cNo_elems);
	//setXY(6,10);
	sendStrXY(6,10,str1);
}

